"""
Description
-----------
This small Python package allows to simply draw upper air sounding on a Skew-T diagramm using
``metpy`` library.

License
-------
All this project is under GNU General Public License v3.0+ (GPLv3+).
Please see the LICENSE file for further informations.

Links
-----
* Home page: https://github.com/Shadow15510/cra
* Documentation: http://cra.rtfd.io/
"""

__version__ = "1.1.4"
